# coding: utf-8

from collections import OrderedDict
import logging
import pkg_resources

from tionix_client.helpers import PACKAGES

logger = logging.getLogger(__name__)


class TionixModuleInfo(object):
    def __init__(self, surrogate_name, pip_name=None, version=None,
                 created_at=None, expire_at=None, lic=None):
        self.surrogate_name = surrogate_name
        self.pip_name = pip_name
        self.version = version
        self.created_at = created_at or '-'
        self.expire_at = expire_at or '-'
        self.lic_id = get_lic_id(lic)
        self.verified = None

        if lic:
            self.expire_at = lic.expire_at
            self.created_at = lic.created_at
            self.verified = lic.verified

        if self.version is None:
            self.verified = None

    def to_row(self):
        version = self.version or '-'
        if self.verified is None:
            verified = '-'
        else:
            verified = u'Valid' if self.verified is True else u'Not valid'
        return [
            unicode(self.surrogate_name),
            unicode(version),
            unicode(self.created_at),
            unicode(self.expire_at),
            unicode(verified)
        ]


def get_lic_id(lic=None):
    lic_id = u'-'

    if lic:
        lic_id = unicode(lic.unique_id)

    return lic_id


def get_package_version(package_name):
    version = None
    try:
        version = pkg_resources.get_distribution(package_name).version
    except pkg_resources.DistributionNotFound:
        logger.warning(u"Package '{0}' is not installed".format(package_name))
    except Exception as err:
        logger.warning(u"Failed to get package '{0}' version: {1}".format(
            package_name, err))
    return version


def get_node_control_info(nova_client):
    surrogate_name = PACKAGES.TIONIX_NC
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    lic = {}
    if version and nova_client:
        try:
            lic = nova_client.nc_licenses.get_node_control_info()
        except Exception as err:
            logger.warning(
                u'Unable to get {0} '
                u'license info: {1}'.format(surrogate_name, err))

    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version,
                            lic=lic)


def get_cloud_control_info(nova_client):
    surrogate_name = PACKAGES.TIONIX_CC
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    lic = {}
    if version and nova_client:
        try:
            lic = nova_client.cc_licenses.get_cloud_control_info()
        except Exception as err:
            logger.warning(
                u'Unable to get {0} '
                u'license info: {1}'.format(surrogate_name, err))

    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version,
                            lic=lic)


def get_monitor_info(nova_client):
    surrogate_name = PACKAGES.TIONIX_Monitor
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    lic = {}
    if version and nova_client:
        try:
            lic = nova_client.monitor_licenses.get_monitor_info()
        except Exception as err:
            logger.warning(
                u'Unable to get {0} '
                u'license info: {1}'.format(surrogate_name, err))

    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version,
                            lic=lic)


def get_vdi_server_info(nova_client):
    surrogate_name = PACKAGES.TIONIX_VDIServer
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    lic = {}
    if version and nova_client:
        try:
            lic = nova_client.vdi_server_licenses.get_vdi_server_info()
        except Exception as err:
            logger.warning(
                u'Unable to get {0} '
                u'license info: {1}'.format(surrogate_name, err))

    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version,
                            lic=lic)


def get_scheduler_info():
    surrogate_name = PACKAGES.TIONIX_Scheduler
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version)


def get_dashboard_info():
    surrogate_name = PACKAGES.TIONIX_Dashboard
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version)


def get_scheduler_client_info():
    surrogate_name = PACKAGES.TIONIX_SchedulerClient
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version)


def get_tionix_client_info():
    surrogate_name = PACKAGES.TIONIX_Client
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version)


def get_tionix_vdi_client_info():
    surrogate_name = PACKAGES.TIONIX_VDIClient
    pip_name = PACKAGES.pip_names[surrogate_name]
    version = get_package_version(pip_name)
    return TionixModuleInfo(surrogate_name, pip_name=pip_name, version=version)


def get_tionix_modules_info(nova_client, exclude_modules=None):

    exclude_modules = exclude_modules or []

    modules_map = OrderedDict([
        (PACKAGES.TIONIX_CC, lambda: get_cloud_control_info(nova_client)),
        (PACKAGES.TIONIX_NC, lambda: get_node_control_info(nova_client)),
        (PACKAGES.TIONIX_Monitor, lambda: get_monitor_info(nova_client)),
        (PACKAGES.TIONIX_VDIServer, lambda: get_vdi_server_info(nova_client)),
        (PACKAGES.TIONIX_Scheduler, get_scheduler_info),
        (PACKAGES.TIONIX_Dashboard, get_dashboard_info),
        (PACKAGES.TIONIX_SchedulerClient, get_scheduler_client_info),
        (PACKAGES.TIONIX_Client, get_tionix_client_info),
        (PACKAGES.TIONIX_VDIClient, get_tionix_vdi_client_info)
    ])
    return [get_module_info()
            for module, get_module_info in modules_map.items()
            if module not in exclude_modules]


def get_modules_info_data(nova_client):
    headers = (
        u'Module name',
        u'Version',
        u'License start date',
        u'License end date',
        u'License status')

    modules = get_tionix_modules_info(nova_client)
    items = [module.to_row() for module in modules]

    return headers, items
