# coding: utf-8

import logging

from keystoneclient.exceptions import Unauthorized
from keystoneclient.client import Client as KeystoneClient

from tionix_client.settings import KEYSTONE

logger = logging.getLogger(__name__)


# pylint: disable=too-many-arguments
def get_keystone_client(token=None, username=None, user_id=None, password=None,
                        domain=None, domain_id=None, tenant=None,
                        tenant_id=None, project_domain_name=None,
                        project_domain_id=None,
                        user_domain_name=None, user_domain_id=None,
                        trust_id=None, auth_url=None,
                        auth_version=None):
    """Function to get keystone client by provided params.

    :param token: auth token
    :param user_id: User ID for authentication
    :param username: Username for authentication
    :param password: Password for authentication
    :param domain: Domain name for domain scoping
    :param domain_id: Domain ID for domain scoping
    :param tenant: tenant name
    :param tenant_id: tenant id
    :param project_domain_id: Project's domain ID for project scoping
    :param project_domain_name: Project's domain name for project scoping
    :param user_domain_id: User's domain ID for authentication
    :param user_domain_name: User's domain name for authentication
    :param trust_id: Trust ID for trust scoping
    :param auth_url: service endpoint for authorization
    :param auth_version: plugin version for authorization
    :return: keystone client
    """
    params = {
        'auth_url': auth_url or KEYSTONE['auth_url'],
        'version': auth_version or KEYSTONE['auth_version'],
        'token': token,
        'username': username,
        'user_id': user_id,
        'password': password,
        'trust_id': trust_id,
        'domain_id': domain_id,
        'domain_name': domain,
        'tenant_id': tenant_id,
        'tenant_name': tenant,
    }

    if username:
        if user_domain_id:
            params['user_domain_id'] = user_domain_id
        else:
            params['user_domain_name'] = (
                user_domain_name or KEYSTONE['user_domain_name'])

    if tenant:
        if project_domain_id:
            params['project_domain_id'] = project_domain_id
        else:
            params['project_domain_name'] = (
                project_domain_name or KEYSTONE['project_domain_name'])

    if not (token or (username or user_id) and password):
        raise AttributeError(
            u'Need a token or username and password for Keystone authenticate')

    try:
        keystone_cl = KeystoneClient(**params)
        keystone_cl.session.auth = keystone_cl
        if not keystone_cl.authenticate():
            raise AttributeError(u'KeystoneClient was not authenticated')

    except Unauthorized:
        msg = u'Parameters used for authentication are invalid: \n'
        msg += '\n'.join(unicode(k) + ': "' + unicode(v) + '"'
                         for k, v in params.iteritems())

        raise Unauthorized(msg)

    return keystone_cl


def get_tenants_manager(keystone_cl):
    try:
        manager = keystone_cl.projects
    except AttributeError:
        manager = keystone_cl.tenants
    return manager
