# coding: utf-8

import logging
import pkg_resources

from novaclient.client import Client as NovaClient

from tionix_client import settings
from tionix_client.os_clients.base import ClientProxy
from tionix_client.os_clients.keystone_client import get_keystone_client

logger = logging.getLogger(__name__)

nova_client_extensions = [
    ep.resolve() for ep in
    pkg_resources.iter_entry_points('novaclient.extension')]


# pylint: disable=too-many-arguments
def get_nova_client(token=None, username=None, password=None,
                    tenant=None, tenant_id=None, project_domain_name=None,
                    user_domain_name=None, trust_id=None,
                    auth_url=None, auth_version=None,
                    compute_service_name=None, timeout=None,
                    keystone_client=None):
    """Function to get nova client by ks_client.

    :param token: auth token
    :param username: auth name
    :param password: auth password
    :param tenant: tenant name
    :param tenant_id: tenant id
    :param trust_id: Trust ID for trust scoping
    :param project_domain_name: project_domain_name
    :param user_domain_name: user_domain_name
    :param compute_service_name: compute_service_name
    :param auth_url: auth_url
    :param auth_version: auth_version
    :param timeout: timeout
    :param keystone_client: keystone_client
    """
    if not keystone_client:
        keystone_client = get_keystone_client(
            token=token,
            username=username,
            password=password,
            tenant=tenant,
            tenant_id=tenant_id,
            user_domain_name=user_domain_name,
            project_domain_name=project_domain_name,
            auth_url=auth_url,
            auth_version=auth_version,
            trust_id=trust_id,
        )
    nova_url = keystone_client.service_catalog.url_for(
        service_type=(
            compute_service_name or settings.KEYSTONE['compute_service_name'])
    )

    try:
        nova_client = NovaClient(
            settings.NOVA_VERSION,
            auth_token=keystone_client.auth_token,
            auth_url=keystone_client.auth_url,
            bypass_url=nova_url,
            extensions=nova_client_extensions,
            timeout=timeout)
    except Exception as err:
        logger.error(unicode(err))
        raise err

    return nova_client


class NovaClientProxy(ClientProxy):

    def init_client(self):
        return get_nova_client(token=self.token, tenant_id=self.tenant_id)

    def refresh_client(self):
        self.client = get_nova_client(
            username=settings.KEYSTONE['auth_user'],
            password=settings.KEYSTONE['auth_password'],
            trust_id=self.trust_id,
            tenant_id=self.tenant_id)


def get_trusted_nova_client(token=None, tenant_id=None):

    return NovaClientProxy(token, tenant_id)
